<?php

	/**
	 * Klasse für die individualisierung von PDF Dokumenten
	 */
	class wpsg_mod_downloadplus extends wpsg_mod_basic {

	    var $lizenz = 1;
	    var $id = 101;
	    var $version = "9.9.9";
	    var $hilfeURL = 'http://wpshopgermany.de/?p=1426';

	    var $valid_ending = array('pdf');
	    var $barCodeFonts = array('EAN13', 'Code128', 'UPC_A');

	    var $arMessage = Array();
	    var $arError = Array();

	    /**
	     * Costructor
	     */
	    public function __construct() {

	        parent::__construct();

	        $this->name = __('DownloadPlus', 'wpsg');
	        $this->group = __('Produkte', 'wpsg');
	        $this->desc = __('Ermöglicht den Verkauf von angepassten PDF Dokumenten.', 'wpsg');

	    } // public function __construct()

	    public function install() {

	        require_once(WPSG_PATH_WP.'/wp-admin/includes/upgrade.php');

	        /**
	         * Tabelle für die Anpassungen der PDF Dokumente
	         */
	        $sql = "CREATE TABLE ".WPSG_TBL_PDFINDIV." (
				id INT(11) NOT NULL AUTO_INCREMENT,				
				o_id INT(11) NOT NULL,
				p_key VARCHAR(255) NOT NULL,
				product_index INT(11) NOT NULL,
				file VARCHAR(255) NOT NULL,
				out_file VARCHAR(255) NOT NULL,
				downloadnr VARCHAR(500) NOT NULL,
				scheduled DATETIME NOT NULL,
				done DATETIME NOT NULL,		
				counter INT(11) NOT NULL,		
				fielddata TEXT NOT NULL,
				mode INT(1) NOT NULL COMMENT 'Individualisierungsmodus im Produkt eingestellt',
				template INT(11) NOT NULL COMMENT 'Link zu WPSG_TBL_DP_TEMPLATES, im Produkt eingestellt',
				INDEX o_id (o_id),
				INDEX p_key (p_key),
				INDEX product_index (product_index),
			    PRIMARY KEY  (id)
			) ENGINE=MyISAM  DEFAULT CHARSET=utf8 AUTO_INCREMENT=1 ;";

	        dbDelta($sql);

	        $sql = "CREATE TABLE ".WPSG_TBL_DP_TEMPLATES." (
				id INT(11) NOT NULL AUTO_INCREMENT,				
				deleted INT(1) NOT NULL COMMENT '1 wenn gelöscht',
				name VARCHAR(255) COMMENT 'Name des Templates' NOT NULL,
				defaulttemplate INT(1) COMMENT '1 beim Standard Tempalte' NOT NULL,
				INDEX deleted (deleted),					
			    PRIMARY KEY  (id)
			) ENGINE=MyISAM  DEFAULT CHARSET=utf8 AUTO_INCREMENT=1 ;";

	        dbDelta($sql);

	        $sql = "CREATE TABLE ".WPSG_TBL_DP_TEMPLATES_FIELD." (
				id INT(11) NOT NULL AUTO_INCREMENT,				
				template_id INT(11) COMMENT 'Link zu WPSG_TBL_TEMPLATES' NOT NULL,		
				deleted INT(1) NOT NULL COMMENT '1 wenn gelöscht',			
				x INT(11) NOT NULL COMMENT 'X - Wert',
				y INT(11) NOT NULL COMMENT 'Y - Wert',
				align INT(1) NOT NULL COMMENT 'Zentriert',
				bg INT(1) NOT NULL COMMENT 'Hintergrund',
				color VARCHAR(6) NOT NULL COMMENT 'Hex Farbe',
				alpha DOUBLE(10,2) NOT NULL COMMENT 'Transparenzwert',
				angle INT(11) NOT NULL COMMENT 'Winkel',
				fontsize INT(11) NOT NULL COMMENT 'Schriftgröße',
				aktiv INT(1) NOT NULL COMMENT 'Aktiv',
				text VARCHAR(1000) NOT NULL COMMENT 'Freitext',
				font VARCHAR(255) NOT NULL COMMENT 'Schriftart',
				INDEX template_id (template_id),
				INDEX deleted (deleted),
			    PRIMARY KEY  (id)
			) ENGINE=MyISAM  DEFAULT CHARSET=utf8 AUTO_INCREMENT=1 ;";

	        dbDelta($sql);

	        $sql = "CREATE TABLE ".WPSG_TBL_VARIANTS_VARI." (
				template_id int(11) NOT NULL COMMENT 'Template für Downloadplus',
				INDEX template_id (template_id),				
				KEY variant_id (variant_id),
			) ENGINE=MyISAM DEFAULT CHARSET=utf8 AUTO_INCREMENT=1;";

	        dbDelta($sql);

	        // Abwärtskombatibilität mit DLPlus
	        if ($this->shop->isMultiBlog()) {

	            $path = WP_CONTENT_DIR.'/'.WPSG_MB_UPLOADS.'/wpsg/wpsg_dlplus/';
	            $new_path = WP_CONTENT_DIR.'/'.WPSG_MB_UPLOADS.'/wpsg/wpsg_pdfprodukte/';

	        } else {

	            $path = WP_CONTENT_DIR.'/uploads/wpsg/wpsg_dlplus/';
	            $new_path = WP_CONTENT_DIR.'/uploads/wpsg/wpsg_pdfprodukte/';

	        }

	        if (file_exists($path)) {

	            rename($path, $new_path);

	        }

	        // Alte Serialisierte Texte in ein Standard Template migrieren
		    $arOldText = $this->shop->get_option('wpsg_mod_downloadplus_text');

	    	if (wpsg_isSizedArray($arOldText)) {

	    		$this->db->Query("UPDATE `".WPSG_TBL_DP_TEMPLATES."` SET `defaulttemplate` = '0' ");

	    		$template_id = $this->db->ImportQuery(WPSG_TBL_DP_TEMPLATES, [
	    			'name' => 'Standard',
				    'deleted' => '0',
				    'defaulttemplate' => '1'
			    ]);

	    		foreach ($arOldText as $field) {

	    			$this->db->ImportQuery(WPSG_TBL_DP_TEMPLATES_FIELD, [
	    				'template_id' => wpsg_q($template_id),
					    'deleted' => '0',
					    'x' => wpsg_q($field['x']),
					    'y' => wpsg_q($field['y']),
					    'align' => wpsg_q($field['align']),
					    'bg' => wpsg_q($field['bg']),
					    'color' => wpsg_q($field['color']),
					    'alpha' => wpsg_q($field['alpha']),
					    'angle' => wpsg_q($field['angle']),
					    'fontsize' => wpsg_q($field['fontsize']),
					    'aktiv' => wpsg_q($field['aktiv']),
					    'text' => wpsg_q($field['text']),
					    'font' => wpsg_q($field['font'])
				    ]);

				}

				// In dem Zuge versuche ich alle Produkte mit Downloadfiles als Downloadprodukt zu markieren
			    $arProductID = $this->db->fetchAssocField("SELECT `id` FROM `".WPSG_TBL_PRODUCTS."` ");

	    		foreach ($arProductID as $product_id) {

	    			$arFiles = $this->getFiles($product_id);

	    			if (sizeof($arFiles) > 0) {

						$oProduct = wpsg_product::getInstance($product_id);
						$oProduct->setMeta('mod_downloadplus_set', "1");

				    }

				}

				$this->shop->update_option('wpsg_mod_downloadplus_text', false);

		    }

	        // Standardwerte
	        $this->shop->checkDefault('wpsg_mod_downloadplus_filename', '%order_id%-%filename%');
	        $this->shop->checkDefault('wpsg_mod_downloadplus_shipping', '1');
	        $this->shop->checkDefault('wpsg_mod_downloadplus_memorylimit', '1');
	        $this->shop->checkDefault('wpsg_mod_downloadplus_timelimit', '1');
	        $this->shop->checkDefault('wpsg_mod_downloadplus_numberformat', '%download_nr%');
	        $this->shop->checkDefault('wpsg_mod_downloadplus_startnumber', '1');
	        $this->shop->checkDefault('wpsg_mod_downloadplus_autosend', '1');
	        $this->shop->checkDefault('wpsg_mod_downloadplus_indiv_mode', '1');

	    } // public function install()

	    public function delOrder($order_id)
	    {

	        $this->db->Query("DELETE FROM `".WPSG_TBL_PDFINDIV."` WHERE `o_id` = '".wpsg_q($order_id)."'");

	        // Individualisierte Dateien löschen
	        $path = $this->getPDFIndivPath($order_id); wpsg_rrmdir($path);

	    } // public function delOrder($order_id)

	    public function wpsg_enqueue_scripts()
	    {

	        if (is_admin() && preg_match('/wpsg/', wpsg_getStr($_REQUEST['page'])))
	        {

	            wp_enqueue_script('wpsg_ajaxupload', $this->shop->getRessourceURL('js/ajaxupload.js'));

	        }

	    } // function wpsg_enqueue_scripts()

	    public function admin_emailconf() {

	        echo wpsg_drawEMailConfig(
	            'pdfdownloadfiles',
	            __('PDFdownload E-Mail', 'wpsg'),
	            __('E-Mail mit Links zu den PDFs die an den Kunden geht.', 'wpsg'));

	        echo wpsg_drawEMailConfig(
	            'pdfdownloadcronmail',
	            __('PDFdownload Cron E-Mail', 'wpsg'),
	            __('E-Mail mit Informationen zu der PDF Individualisierung die vom Cron verschickt wird. (An Admin!).', 'wpsg'),
	            true);

	    } // public function admin_emailconf()

	    public function admin_emailconf_save()
	    {

	        wpsg_saveEMailConfig("pdfdownloadfiles");
	        wpsg_saveEMailConfig("pdfdownloadcronmail");

	    } // public function admin_emailconf_save()

	    public function settings_save() {

		    $arTemplateID = [];
		    $arFieldID = [];

		    $_REQUEST['template'] = json_decode(($_REQUEST['template']??[]), true);

		    if (!is_array($_REQUEST['template'])) throw new \Exception(__('Etwas stimmte mit den Templatedaten nicht.'));

		    foreach ($_REQUEST['template'] as $template) {

		    	$template_id = intval($template['id']);

		    	$db_template_data = [
		    		'name' => wpsg_q($template['name']),
				    'defaulttemplate' => wpsg_q($template['defaulttemplate'])
			    ];

		    	if ($template_id > 0) {

		    		$this->db->UpdateQuery(WPSG_TBL_DP_TEMPLATES, $db_template_data, " `id` = '".wpsg_q($template_id)."' ");

			    } else {

		    		$template_id = $this->db->ImportQuery(WPSG_TBL_DP_TEMPLATES, $db_template_data);

			    }

		    	if (wpsg_isSizedString($template['file_data'])) {

		    		$path = $this->getPDFTemplatePath($template_id);
		    		wpsg_clearDirectory($path);

		    		$file_data = $template['file_data'];
		    		$mime = 'data:application/pdf;base64,';

		    		if (substr($file_data, 0, strlen($mime)) !== $mime) throw new \Exception(__('Ungültiger Dateityp', 'wpsg'));

		    		file_put_contents($path.$template['file'], base64_decode(substr($file_data, strlen($mime))));

			    } else if ($template['file_delete'] === true) {

		    		$path = $this->getPDFTemplatePath($template_id);
		    		wpsg_clearDirectory($path);

			    }

		    	foreach ($template['arField'] as $field) {

		    		$field_id = intval($field['id']);

		    		$db_field_data = [
						'x' => wpsg_q($field['x']),
					    'y' => wpsg_q($field['y']),
					    'align' => wpsg_q($field['align']),
					    'bg' => wpsg_q($field['bg']),
					    'color' => wpsg_q($field['color']),
					    'alpha' => wpsg_q(wpsg_tf($field['alpha'])),
					    'angle' => wpsg_q($field['angle']),
					    'fontsize' => wpsg_q($field['fontsize']),
					    'aktiv' => wpsg_q($field['aktiv']),
					    'text' => wpsg_q($field['text']),
					    'font' => wpsg_q($field['font']),
					    'template_id' => wpsg_q($template_id)
				    ];

		    		if ($field_id > 0) {

		    			$this->db->UpdateQuery(WPSG_TBL_DP_TEMPLATES_FIELD, $db_field_data, " `id` = '".wpsg_q($field_id)."' ");

 				    } else {

		    			$field_id = $this->db->ImportQuery(WPSG_TBL_DP_TEMPLATES_FIELD, $db_field_data);

				    }

		    		$arFieldID[] = $field_id;

			    }

		    	$arTemplateID[] = $template_id;

		    }

		    if (sizeof($arTemplateID) > 0) $this->db->UpdateQuery(WPSG_TBL_DP_TEMPLATES, ['deleted' => '1'], " `id` NOT IN (".wpsg_q(implode(',', $arTemplateID)).") ");
		    else $this->db->UpdateQuery(WPSG_TBL_DP_TEMPLATES, ['deleted' => '1'], " 1 ");

		    if (sizeof($arFieldID) > 0) $this->db->UpdateQuery(WPSG_TBL_DP_TEMPLATES_FIELD, ['deleted' => '1'], " `id` NOT IN (".wpsg_q(implode(',', $arFieldID)).") ");
		    else $this->db->UpdateQuery(WPSG_TBL_DP_TEMPLATES_FIELD, ['deleted' => '1'], " 1 ");

	        $this->shop->update_option('wpsg_mod_downloadplus_filename', $_REQUEST['wpsg_mod_downloadplus_filename'], false, false, WPSG_SANITIZE_TEXTFIELD);
	        $this->shop->update_option('wpsg_mod_downloadplus_indiv_mode', $_REQUEST['wpsg_mod_downloadplus_indiv_mode'], false, false, WPSG_SANITIZE_VALUES, ['0', '1', '2']);
	        $this->shop->update_option('wpsg_mod_downloadplus_autosend', $_REQUEST['wpsg_mod_downloadplus_autosend'], false, false, WPSG_SANITIZE_CHECKBOX);
	        $this->shop->update_option('wpsg_mod_downloadplus_shipping', $_REQUEST['wpsg_mod_downloadplus_shipping'], false, false, WPSG_SANITIZE_CHECKBOX);
	        $this->shop->update_option('wpsg_mod_downloadplus_einsplusx', $_REQUEST['wpsg_mod_downloadplus_einsplusx'], false, false, WPSG_SANITIZE_CHECKBOX);
	        $this->shop->update_option('wpsg_mod_downloadplus_accepted', $_REQUEST['wpsg_mod_downloadplus_accepted'], false, false, WPSG_SANITIZE_VALUES, ['0', '1', '2']);
	        $this->shop->update_option('wpsg_mod_downloadplus_days', $_REQUEST['wpsg_mod_downloadplus_days'], false, false, WPSG_SANITIZE_TEXTFIELD);
	        $this->shop->update_option('wpsg_mod_downloadplus_downloads', $_REQUEST['wpsg_mod_downloadplus_downloads'], false, false, WPSG_SANITIZE_TEXTFIELD);
	        $this->shop->update_option('wpsg_mod_downloadplus_cleanraiddownloads', $_REQUEST['wpsg_mod_downloadplus_cleanraiddownloads'], false, false, WPSG_SANITIZE_TEXTFIELD);
	        $this->shop->update_option('wpsg_mod_downloadplus_cleanolddownloads', $_REQUEST['wpsg_mod_downloadplus_cleanolddownloads'], false, false, WPSG_SANITIZE_TEXTFIELD);
	        $this->shop->update_option('wpsg_mod_downloadplus_pdfpath', $_REQUEST['wpsg_mod_downloadplus_pdfpath'], false, false, WPSG_SANITIZE_TEXTFIELD);
	        $this->shop->update_option('wpsg_mod_downloadplus_pdfindivpath', $_REQUEST['wpsg_mod_downloadplus_pdfindivpath'], false, false, WPSG_SANITIZE_TEXTFIELD);
	        $this->shop->update_option('wpsg_mod_downloadplus_range', $_REQUEST['wpsg_mod_downloadplus_range'], false, false, WPSG_SANITIZE_VALUES, ['0', '1']);
	        $this->shop->update_option('wpsg_mod_downloadplus_range_von', $_REQUEST['wpsg_mod_downloadplus_range_von'], false, false, WPSG_SANITIZE_TEXTFIELD);
	        $this->shop->update_option('wpsg_mod_downloadplus_range_bis', $_REQUEST['wpsg_mod_downloadplus_range_bis'], false, false, WPSG_SANITIZE_TEXTFIELD);
	        $this->shop->update_option('wpsg_mod_downloadplus_masterpass', $_REQUEST['wpsg_mod_downloadplus_masterpass'], false, false, WPSG_SANITIZE_TEXTFIELD);
	        $this->shop->update_option('wpsg_mod_downloadplus_openpass', $_REQUEST['wpsg_mod_downloadplus_openpass'], false, false, WPSG_SANITIZE_TEXTFIELD);
	        $this->shop->update_option('wpsg_mod_downloadplus_securitysave', $_REQUEST['wpsg_mod_downloadplus_securitysave'], false, false, WPSG_SANITIZE_CHECKBOX);
	        $this->shop->update_option('wpsg_mod_downloadplus_securityprint', $_REQUEST['wpsg_mod_downloadplus_securityprint'], false, false, WPSG_SANITIZE_CHECKBOX);
	        $this->shop->update_option('wpsg_mod_downloadplus_securitymod', $_REQUEST['wpsg_mod_downloadplus_securitymod'], false, false, WPSG_SANITIZE_CHECKBOX);
	        $this->shop->update_option('wpsg_mod_downloadplus_securitynote', $_REQUEST['wpsg_mod_downloadplus_securitynote'], false, false, WPSG_SANITIZE_CHECKBOX);
	        $this->shop->update_option('wpsg_mod_downloadplus_format', $_REQUEST['wpsg_mod_downloadplus_format'], false, false, WPSG_SANITIZE_VALUES, ['0', '1']);
	        $this->shop->update_option('wpsg_mod_downloadplus_memorylimit', $_REQUEST['wpsg_mod_downloadplus_memorylimit'], false, false, WPSG_SANITIZE_CHECKBOX);
	        $this->shop->update_option('wpsg_mod_downloadplus_emptypdf', $_REQUEST['wpsg_mod_downloadplus_emptypdf'], false, false, WPSG_SANITIZE_CHECKBOX);
	        $this->shop->update_option('wpsg_mod_downloadplus_timelimit', $_REQUEST['wpsg_mod_downloadplus_timelimit'], false, false, WPSG_SANITIZE_CHECKBOX);
	        $this->shop->update_option('wpsg_mod_downloadplus_numberformat', $_REQUEST['wpsg_mod_downloadplus_numberformat'], false, false, WPSG_SANITIZE_TEXTFIELD);
	        $this->shop->update_option('wpsg_mod_downloadplus_startnumber', $_REQUEST['wpsg_mod_downloadplus_startnumber'], false, false, WPSG_SANITIZE_TEXTFIELD);
	        $this->shop->update_option('wpsg_mod_downloadplus_attachement', $_REQUEST['wpsg_mod_downloadplus_attachement'], false, false, WPSG_SANITIZE_CHECKBOX);

	    } // public function settings_save()

	    public function settings_edit() {

	        $font_path = WPSG_PATH_LIB.'FPDF_1.81/font/';

	        $arFontFiles = scandir($font_path);

	        foreach ($arFontFiles as $k => $v) {

	            $file = $font_path.$v;

	            if (is_file($file) && file_exists($file) && preg_match('/(.*)\.php$/i', $v)) {

	                $this->shop->view['wpsg_mod_downloadplus']['arFonts'][$v] = ucfirst(preg_replace('/\.php$/i', '', $v));

	            }

	        }

	        $this->shop->view['wpsg_mod_downloadplus']['barCodeFonts'] = $this->barCodeFonts;

	        // Barcode
	        $this->shop->view['wpsg_mod_downloadplus']['arFonts']['EAN13'] = 'EAN 13';
	        $this->shop->view['wpsg_mod_downloadplus']['arFonts']['Code128'] = 'Code 128';
	        $this->shop->view['wpsg_mod_downloadplus']['arFonts']['UPC_A'] = 'UPC-A';

	        $this->shop->render(WPSG_PATH_VIEW.'/mods/mod_downloadplus/settings_edit.phtml');

	    } // public function settings_edit()

	    public function produkt_ajax() {

			$product_id = intval($_REQUEST['edit_id']);

			$file = ''; if (isset($_REQUEST['file'])) $file = rawurldecode($_REQUEST['file']);

	        if ($_REQUEST['cmd'] === 'upload_file') {

	            $this->fileUpload($product_id);

	        } else if ($_REQUEST['cmd'] === 'pdffiles_list') {

				echo $this->renderFilesList($product_id);

				exit;

	        } else if ($_REQUEST['cmd'] === 'delete_file') {

	            unlink($this->getPDFPath($product_id).'/'.$file);

	            echo '1'; exit;

	        } else if ($_REQUEST['cmd'] === 'download_file') {

	            wpsg_ob_end_clean();

	            header('Content-Type: application/pdf');
	            header('Content-Disposition: inline; filename="'.$file.'"');
	            header('Cache-Control: private, max-age=0, must-revalidate');
	            header('Pragma: public');

	            echo file_get_contents($this->getPDFPath($product_id).'/'.$file);
	            exit;

	        }

	    } // public function produkt_ajax()

	    public function produkt_save(&$produkt_id) {

	        if (wpsg_isSizedArray($_FILES['wpsg_mod_downloadplus_file']) && file_exists($_FILES['wpsg_mod_downloadplus_file']['tmp_name'])) {

	            $bReturn = $this->fileUpload($produkt_id, false);

	            if ($bReturn !== '1') {

	                $this->shop->addBackendError($bReturn);

	            } else {

	                $this->shop->addBackendMessage(__('Datei für das Downloadplus Modul erfolgreich hochgeladen.', 'wpsg'));

	            }

	        }

	        $oProduct = wpsg_product::getInstance($produkt_id);

	        $oProduct->setMeta('mod_downloadplus_set', $_REQUEST['mod_downloadplus_set']);
	        $oProduct->setMeta('mod_downloadplus_template', $_REQUEST['mod_downloadplus_template']);
	        $oProduct->setMeta('mod_downloadplus_mode', $_REQUEST['mod_downloadplus_mode']);

	        if ($this->shop->hasMod('wpsg_mod_productvariants')) {

	        	$oProduct->setMeta('mod_downloadplus_vari_template', json_encode($_REQUEST['vari_template']??[]));

	        }

	    } // public function produkt_save(&$produkt_id)

	    /*
	     * zeigt ein Upload-Formular im linken Bereich des Produktbackend an
	    */
	    public function product_addedit_content(&$product_content, &$product_data) {

	        if (isset($_REQUEST['wpsg_lang']) || !wpsg_isSizedInt($product_data['id'])) return;

	        if ($product_data['id'] > 0) {

	        	/** @var wpsg_product $oProduct */
	        	$oProduct = wpsg_product::getInstance(intval($product_data['id']));

	            $this->shop->view['wpsg_mod_downloadplus'] = [
	            	'set' => $oProduct->getMeta('mod_downloadplus_set', false, '0'),
	            	'data' => $product_data,
		            'mode' => $oProduct->getMeta('mod_downloadplus_mode', false, '0'),
		            'template' => $oProduct->getMeta('mod_downloadplus_template', false, '0'),
		            'filesList' => $this->renderFilesList($product_data['id'])
	            ];

	        }

	        if ($this->shop->hasMod('wpsg_mod_productvariants')) {

	        	$this->shop->view['wpsg_mod_downloadplus']['vari_template'] = json_decode($oProduct->getMeta('mod_downloadplus_vari_template'), true);

	        }

	        $this->shop->view['wpsg_mod_downloadplus']['arTemplates'] = $this->db->fetchAssocField("
	            SELECT
	                T.`id`,
	            	CONCAT(T.`name`, ' (', (SELECT COUNT(*) FROM `".WPSG_TBL_DP_TEMPLATES_FIELD."` AS F WHERE F.`template_id` = T.`id`), ' ".__('Felder', 'wpsg')."', IF(T.`defaulttemplate` = '1',', Standard',''), ')') AS `label`
	            FROM
					`".WPSG_TBL_DP_TEMPLATES."` AS T
				WHERE
					T.`deleted` != '1'
	        ", "id", "label");

	        $product_content['wpsg_mod_downloadplus'] = array(
	            'title' => __('DownloadPlus Produkt', 'wpsg'),
	            'content' => $this->shop->render(WPSG_PATH_VIEW.'/mods/mod_downloadplus/produkt_addedit_sidebar.phtml', false)
	        );

	    } //public function product_addedit_content(&$product_content, &$product_data)

	    public function setOrderStatus($order_id, $status_id, $inform) {

	        if ($this->shop->get_option('wpsg_mod_downloadplus_indiv_mode') === '1' && $status_id == 100) { // Nach Zahlung planen

	            $this->indivOrder($order_id);

	        }

	    } // public function setOrderStatus($order_id, $status_id, $inform)

	    public function basket_save_done(&$order_id, &$kunde_id, &$oBasket)
	    {

	        if ($this->shop->get_option('wpsg_mod_downloadplus_indiv_mode') != '2') return;

	        $this->indivOrder($order_id);

	    } // public function basket_save_done(&$order_id, &$kunde_id, &$oBasket)

	    /**
	     * Zeichnet die Dateiliste der PDFs
	     */
	    private function renderFilesList($produkt_id) {

	        $this->shop->view['files'] = $this->getFiles($produkt_id, true);
	        return $this->shop->render(WPSG_PATH_VIEW.'/mods/mod_downloadplus/filelist.phtml', false);

	    } // private function renderFilesList($produkt_id)

	    public function basket_toArray(&$produkt, $backend = false, $noMwSt = false)
	    {

	        if ($this->shop->get_option('wpsg_mod_downloadplus_einsplusx') != '1')
	        {

	            if ($this->isPDFProdukt($produkt['id']))
	            {

	                if ($produkt['menge'] != 1)
	                {

	                    // Datei ist ein Downloadprodukt
	                    $produkt['menge'] = '1';

	                    // In der Session auch korrigieren
	                    if (isset($_SESSION['wpsg']['basket']))
	                    {
	                        foreach ($_SESSION['wpsg']['basket'] as $k => $v) { { if ($v['id'] == $produkt['id']) $_SESSION['wpsg']['basket'][$k]['menge'] = 1; } }
	                    }
	                }

	                $produkt['oneOnly'] = true;

	            }

	        }

	    } // public function basket_toArray(&$produkt, $backend = false, $noMwSt = false)

	    public function order_ajax() {

		    $_REQUEST['edit_id'] = intval($_REQUEST['edit_id']??'');
		    $_REQUEST['product_key'] = sanitize_text_field($_REQUEST['product_key']??'');
		    $_REQUEST['product_index'] = intval($_REQUEST['product_index']??'');

	        if ($_REQUEST['cmd'] == 'schedule_file') {

	            $db_indiv = $this->db->fetchRow("
					SELECT 
						`id`
					FROM
						`".WPSG_TBL_PDFINDIV."`
					WHERE
						`file` = '".wpsg_q($_REQUEST['file'])."' AND
						`o_id` = '".wpsg_q($_REQUEST['edit_id'])."' AND
						`product_index` = '".wpsg_q($_REQUEST['product_index'])."' 
				");

	            if ($db_indiv['id'] > 0) {

	                $new_name = $this->getNewName($_REQUEST['file'], $_REQUEST['edit_id'], $_REQUEST['product_key'], $db_indiv['downloadnr'], $_REQUEST['product_index']);

	                $this->db->UpdateQuery(WPSG_TBL_PDFINDIV, array(
	                    "scheduled" => "NOW()",
	                    "out_file" => wpsg_q($new_name)
	                ), "`id` = '".wpsg_q($db_indiv['id'])."'");

	            } else {

	                $download_nr = $this->getDownloadNr($_REQUEST['edit_id'], $_REQUEST['product_key'], $_REQUEST['product_index']);

	                $new_name = $this->getNewName($_REQUEST['file'], $_REQUEST['edit_id'], $_REQUEST['product_key'], $download_nr, $_REQUEST['product_index']);

	                $oProduct = wpsg_product::getInstance($_REQUEST['product_key']);

	                $this->db->ImportQuery(WPSG_TBL_PDFINDIV, array(
	                    "o_id" => wpsg_q($_REQUEST['edit_id']),
	                    "p_key" => wpsg_q($_REQUEST['product_key']),
	                    'product_index' => wpsg_q($_REQUEST['product_index']),
	                    "downloadnr" => wpsg_q($download_nr),
	                    "file" => wpsg_q($_REQUEST['file']),
	                    "scheduled" => "NOW()",
	                    "out_file" => wpsg_q($new_name),
		                'mode' => $oProduct->getMeta('mod_downloadplus_mode'),
		                'template' => $this->getTemplateIdFromProduct($_REQUEST['product_key'])
	                ));

	            }

	            $this->arMessage[] = __('Datei wurde geplant.', 'wpsg');

	            die($this->order_view_sidebar_renderList($_REQUEST['edit_id']));

	        } else if ($_REQUEST['cmd'] == 'reindiv') {

	            $this->indivOrder($_REQUEST['edit_id']);

	            $this->arMessage[] = __('Bestellung wurde geplant.', 'wpsg');

	            die($this->order_view_sidebar_renderList($_REQUEST['edit_id']));

	        }
	        else if ($_REQUEST['cmd'] == 'reset')
	        {

	            $this->db->UpdateQuery(WPSG_TBL_PDFINDIV, array("counter" => "0"), "`o_id` = '".wpsg_q($_REQUEST['edit_id'])."'");

	            $this->arMessage[] = __('Downloadzähler wurden zurückgesetzt.', 'wpsg');

	            die($this->order_view_sidebar_renderList($_REQUEST['edit_id']));

	        }
	        else if ($_REQUEST['cmd'] == 'send_files')
	        {

	            $this->sendFiles($_REQUEST['edit_id']);

	            die($this->order_view_sidebar_renderList($_REQUEST['edit_id']));

	        }
	        else if ($_REQUEST['cmd'] == 'cancelschedule_file')
	        {

	            $this->db->Query("
						DELETE FROM
							`".WPSG_TBL_PDFINDIV."`
						WHERE
							`file` = '".wpsg_q($_REQUEST['file'])."' AND
							`o_id` = '".wpsg_q($_REQUEST['edit_id'])."' AND
							`p_key` = '".wpsg_q($_REQUEST['product_key'])."'
					");

	            $this->arMessage[] = __('Planung wurde storniert.', 'wpsg');

	            die($this->order_view_sidebar_renderList($_REQUEST['edit_id']));

	        }
	        else if ($_REQUEST['cmd'] == 'download_indivfile') {

	            $order_id = intval($_REQUEST['edit_id']);
	            $product_key = $_REQUEST['product_key'];
	            $product_id = intval($this->shop->getProduktID($product_key));

	            check_admin_referer('wpsg-dp-download-'.$order_id.'-'.$product_key);

	            wpsg_ob_end_clean();

	            header('Content-Type: application/pdf');
	            header('Content-Disposition: inline; filename="'.rawurldecode(basename($_REQUEST['file'])).'"');
	            header('Cache-Control: private, max-age=0, must-revalidate');
	            header('Pragma: public');

	            if (isset($_REQUEST['clear'])) die(file_get_contents($this->getPDFIndivPath($order_id).'/clear_'.rawurldecode($_REQUEST['file'])));
	            else die(file_get_contents($this->getPDFIndivPath($order_id).'/'.rawurldecode($_REQUEST['file'])));

	        }

	    } // public function order_ajax()

	    public function order_view($order_id, &$arSidebarArray) {

	        $bPDFProdukt = false;
	        $arFiles = array();

	        // Die arFiles brauch ich um zu bestimmen ob Produkte gekauft wurden, die Downloads enthalten
	        foreach ($this->shop->view['basket']['produkte'] as $k => $v) {

	            $this->shop->view['arProducts'][$k] = $v;

	            if ($this->isPDFProdukt($v['id'])) {

	                $bPDFProdukt = true;

	                $product_id = $this->shop->getProduktID($v['id']);

	                // Hier darf das wpsg_array_merge nicht verwendet werden, da der Index nicht überschrieben werden darf
	                //$arFiles = array_merge($arFiles, $this->getFiles($v['id']));
	                $arFiles_product = $this->getfiles($v['id']);

	                foreach ($arFiles_product as $file_product) {

	                    $arFiles[] = array($product_id, $file_product);

	                }

	            }

	        }

	        if ($bPDFProdukt && sizeof($arFiles) > 0) {

	            $this->shop->view['order_id'] = $order_id;

	            $arSidebarArray[$this->id] = array(
	                'title' => $this->name,
	                'content' => $this->shop->render(WPSG_PATH_VIEW.'/mods/mod_downloadplus/order_view_sidebar.phtml', false)
	            );

	        }

	    } // public function order_view($order_id, &$arSidebarArray)

	    public function order_view_sidebar_renderList($order_id) {

	        $bPDFProdukt = false;
	        $arFiles = array();

	        $oOrder = wpsg_order::getInstance($order_id);

	        foreach ($oOrder->getOrderProducts() as $oOrderProduct) {

	        	$product_key = $oOrderProduct->getProductKey();
	        	$product_files = $this->getFiles($product_key);

	        	foreach ($product_files as $file) {

	        		$bPDFProdukt = true;
                    $arFiles[] = array($product_key, $file, $oOrderProduct->getProductIndex());

                }

	        }

	        $arIndivIDs = array();

	        if ($bPDFProdukt && wpsg_isSizedArray($arFiles)) {

	            $arFiles = wpsg_array_unique($arFiles);

	            foreach ($arFiles as $k => $v) {

	                $db_indiv = $this->db->fetchRow("
						SELECT 
							* 
						FROM 
							`".WPSG_TBL_PDFINDIV."` 
						WHERE
							`p_key` = '".wpsg_q($v[0])."' AND
							`o_id` = '".wpsg_q($order_id)."' AND
							`product_index` = '".wpsg_q($v[2])."'
					"); // `file` = '".wpsg_q($v[1])."' AND

	                if (!wpsg_isSizedInt($db_indiv['id'])) {

	                	unset($arFiles[$k]); continue;

	                }

	                $clear_file = $this->getPDFIndivPath($order_id).'clear_'.$db_indiv['out_file'];

	                if (file_exists($clear_file)) {

	                    $db_indiv['clear_file'] = true;

	                } else {

	                    $db_indiv['clear_file'] = false;

	                }

	                if ($db_indiv['id'] > 0) {

	                    $arIndivIDs[] = $db_indiv['id'];

	                    $db_indiv['raid'] = false;

	                    if ($this->shop->get_option('wpsg_mod_downloadplus_accepted') == '1' && $db_indiv['counter'] >= $this->shop->get_option('wpsg_mod_downloadplus_downloads')) {

	                        $db_indiv['raid'] = true;

	                    }

	                    if ($this->shop->get_option('wpsg_mod_downloadplus_accepted') == '2' && strtotime($db_indiv['done']) > 0 && time() > (strtotime($db_indiv['done']) + ($this->shop->get_option('wpsg_mod_downloadplus_days') * 86400))) {

	                        $db_indiv['raid'] = true;

	                    }

	                    $arFiles[$k][3] = $db_indiv;

	                } else {

	                    $arFiles[$k][3] = false;

	                }

	            }

	            // Jetzt noch eventuell individualisierte Dokumente laden die in den Produkten gelöscht wurden
	            if (wpsg_isSizedArray($arIndivIDs)) {

	                $arDbIndivDeleted = $this->db->fetchAssoc("SELECT * FROM `".WPSG_TBL_PDFINDIV."` WHERE `id` NOT IN (".implode(",", $arIndivIDs).") AND `o_id` = '".wpsg_q($order_id)."'");

	                foreach ($arDbIndivDeleted as $v) {

	                	$product_id = $this->shop->getProduktID($v['p_key']);

	                	$clear_file = $this->getPDFIndivPath($order_id).'clear_'.$v['out_file'];

		                if (file_exists($clear_file)) {

		                    $v['clear_file'] = true;

		                } else {

		                    $v['clear_file'] = false;

		                }

	                    $arFiles[] = array($product_id, $v['file'], false, $v);

	                }

	            }

	            $this->shop->view['messages'] = $this->arMessage;
	            $this->shop->view['errors'] = $this->arError;
	            $this->shop->view['arFiles'] = $arFiles;

	            $this->shop->render(WPSG_PATH_VIEW.'/mods/mod_downloadplus/order_view_sidebar_renderlist.phtml');

	        }

	    } // public function order_view_sidebar_renderList($order_id)

	    public function addShipping(&$arShipping, $va_active = false)
	    {

	        if ($this->shop->get_option('wpsg_mod_downloadplus_shipping') != '1') return;

	        $arShipping[$this->id] = array(
				'active' => '1',
	            'id' => $this->id,
	            'name' => __('Versand per Mail', 'wpsg'),
	            'price' => 0,
	            'tax_key' => 0
	        );

	    } // public function addShipping(&$arShipping)

	    /**
	     * lädt eine Datei in das entspr. upload Verzeichnis
	     */
	    private function fileUpload($produkt_id, $die = true)
	    {

	        if (file_exists($_FILES['wpsg_mod_downloadplus_file']['tmp_name'][0]))
	        {

	            $ending = strtolower(preg_replace('/(.*)\./', '', $_FILES['wpsg_mod_downloadplus_file']['name'][0]));

	            if (!in_array($ending, $this->valid_ending))
	            {

	                if (!$die) return __('Ungültige Dateiendung.', 'wpsg');
	                else die(__('Ungültige Dateiendung.', 'wpsg'));

	            }
	            else
	            {

	                $uploaddir = $this->getPDFPath($produkt_id);
	                if (!file_exists($uploaddir)) mkdir($uploaddir, 0777, true);

	                $uploadfile = $uploaddir.\sanitize_file_name(basename($_FILES['wpsg_mod_downloadplus_file']['name'][0]));

	                move_uploaded_file($_FILES['wpsg_mod_downloadplus_file']['tmp_name'][0], $uploadfile);

	                if (!$die) return '1';
	                else die('1');


	            }

	        }
	        else
	        {

	            if (!$die) return __('Fehler beim Upload.', 'wpsg');
	            else die(__('Fehler beim Upload.', 'wpsg'));

	        }

	    } // private function fileUpload($produkt_id)

	    /**
	     * Individualisiert das PDF $src_file und speichert es unter $trg_file, Absolute Pfade
	     */
	    private function indivPDF($p_key, $src_file, $trg_file, $order_id, $download_nr, $indiv_id, $product_index, $paintBackground = true) {

			$p_id = $this->shop->getProduktID($p_key);
			$indiv = $this->db->fetchRow("SELECT * FROM `".WPSG_TBL_PDFINDIV."` WHERE `id` = '".wpsg_q($indiv_id)."' ");

			$varName = '';
			$varName_checkbox = ''; // Spezialfall

			if ($this->shop->callMod('wpsg_mod_productvariants', 'isVariantsProduct', array($p_id))) {

				$variInfo = $this->shop->callMod('wpsg_mod_productvariants', 'getVariantenInfoArray', array($p_key));

				if (wpsg_isSizedArray($variInfo)) {

					foreach ($variInfo as $k => $v) {

						if (is_numeric($k)) {

							if ($v['value'] !== 'Nein') $varName_checkbox .= ' '.$v['name'];

						}

					}

					$varName = $variInfo['key'];

				}

			}

			$varName_checkbox = trim($varName_checkbox);

	        if ($this->shop->get_option('wpsg_mod_downloadplus_memorylimit') == '1') ini_set('memory_limit', '2000M');
	        if ($this->shop->get_option('wpsg_mod_downloadplus_timelimit') == '1') set_time_limit(3600);

			require_once WPSG_PATH_LIB.'FPDF_1.81/fpdf.php';
			require_once WPSG_PATH_LIB.'FPDI_2.2.0/autoload.php';
			require_once WPSG_PATH_LIB.'FpdiProtection_2/autoload.php';
	        require_once WPSG_PATH_MOD."mod_downloadplus/wpsg_fpdf.php";

	        $pi = pathinfo($src_file);

	        if (in_array(strtolower($pi['extension']), $this->valid_ending)) {

	            $pdf = new wpsg_dl_fpdf();

	            $pdf->setSourceFile($src_file);

	            $tplidx = $pdf->importPage(1, '/MediaBox');
	            $size = $pdf->getTemplateSize($tplidx);

	            if (!wpsg_isSizedArray($size)) throw new \Exception(__('Konnte PDF nicht lesen', 'wpsg'));

	            $w = $size['width']; $h = $size['height'];
	            $orientation = (($h > $w)?'P':'L');

	            $pdf = new wpsg_dl_fpdf($orientation, "mm", array($w, $h));;

	            $arRights = array();

	            if ($this->shop->get_option("wpsg_mod_downloadplus_securitysave") == "1") $arRights[] = "copy";
	            if ($this->shop->get_option("wpsg_mod_downloadplus_securityprint") == "1") $arRights[] = "print";
	            if ($this->shop->get_option("wpsg_mod_downloadplus_securitymod") == "1") $arRights[] = "modify";
	            if ($this->shop->get_option("wpsg_mod_downloadplus_securitynote") == "1") $arRights[] = "annot-forms";

	            $pdf->SetProtection($arRights, $this->shop->get_option("wpsg_mod_downloadplus_openpass"), $this->shop->get_option("wpsg_mod_downloadplus_masterpass"));

	            $pagecount = $pdf->setSourceFile($src_file);

	            $oProduct = wpsg_product::getInstance($p_key);

	            $arCustomReplace = array(
		            '/%filename%/i' => basename($src_file),
		            '/%filename_clear%/i' => preg_replace('/\.(.*)$/', '', $src_file),
		            '/%filename_ohneext%/i' => preg_replace('/\.(.*)$/', '', basename($src_file)),
		            '/%product_detailname%/i' => $oProduct->getProductName(true),
		            '/%product_name%/i' => $oProduct->getProductName(),
		            '/%product_anr%/i' => $oProduct->getNr(),
		            '/%product_preis%/i' => wpsg_ff($oProduct->getPrice($p_key, $this->shop->getFrontendTaxview())),
		            '/%product_netto_preis%/i' => wpsg_ff($oProduct->getPrice($p_key, WPSG_NETTO)),
		            '/%product_beschreibung%/i' => $oProduct->getShortDescription(),
		            '/%product_varianten_name%/i' => $varName,
		            '/%product_varianten_name_checkbox%/i' => $varName_checkbox,
		            '/%download_nr%/i' => $download_nr
	            );

	            //$arTexte = $this->shop->get_option('wpsg_mod_downloadplus_text');
		        if (intval($indiv['template']) > 0) $arTexte = $this->getTemplateFields(intval($indiv['template']));
		        else $arTexte = $this->getTemplateFields();

		        array_walk($arTexte, function(&$el) {

		        	$el['alpha'] = wpsg_tf($el['alpha']);

		        });

	            $fielddata = array();

	            for ($pagei = 1; $pagei <= $pagecount; $pagei++) {

		            if ($paintBackground) $tplidx = $pdf->importPage($pagei, '/BleedBox');

		            $pdf->addPage();

		            if (wpsg_isSizedArray($arTexte) && $this->shop->get_option("wpsg_mod_downloadplus_range") != "1" || ($pagei >= $this->shop->get_option("wpsg_mod_downloadplus_range_von") && $pagei <= $this->shop->get_option("wpsg_mod_downloadplus_range_bis"))) {

			            foreach ($arTexte as $text_key => $t) {

				            if ($t['bg'] && $t['aktiv']) {

					            $t['text'] = $this->shop->replaceUniversalPlatzhalter($t['text'], $order_id, false, false, $p_id, $arCustomReplace, $product_index);
					            $fielddata['texte'][$text_key] = $t['text'];

					            if ($t['alpha'] > 0) {

						            $pdf->setAlpha($t['alpha']);

					            }

					            $bSetFont = false;

					            if (wpsg_isSizedString($t['font']) && !in_array($t['font'], $this->barCodeFonts)) {

						            $font_file = WPSG_PATH_LIB . 'FPDF_1.81/font/' . $t['font'];

						            if (file_exists($font_file)) {

							            global $name;

							            include $font_file;

							            $bSetFont = true;

							            $pdf->SetFont($name, '', $t['fontsize']);

						            }

					            }

					            if (!$bSetFont) $pdf->SetFont('Arial', 'B', $t['fontsize']);

					            if (in_array($t['font'], $this->barCodeFonts)) {

						            $font_function = $t['font'];

						            $pdf->SetDrawColor(0, 0, 0);
						            $pdf->$font_function($t['x'], $t['y'], $t['text'], $t['fontsize']);

					            } else {

						            $pdf->wpsg_SetTextColor($t["color"]);

						            $pdf->RotatedText(
							            $t['x'], $t['y'],
							            $t['text'],
							            $t['angle'],
							            $t['align']
						            );

						            $pdf->setAlpha(1);

					            }

				            }

			            }

		            }

		            if ($paintBackground) $pdf->useTemplate($tplidx, 0, 0, $w, $h);

		            if ($this->shop->get_option("wpsg_mod_downloadplus_range") != "1" || ($pagei >= $this->shop->get_option("wpsg_mod_downloadplus_range_von") && $pagei <= $this->shop->get_option("wpsg_mod_downloadplus_range_bis"))) {

			            foreach ($arTexte as $text_key => $t) {

				            if (!$t['bg'] && $t['aktiv']) {

					            $t['text'] = $this->shop->replaceUniversalPlatzhalter($t['text'], $order_id, false, false, $p_id, $arCustomReplace, $product_index);
					            $fielddata['texte'][$text_key] = $t['text'];

					            if ($t['alpha'] > 0) $pdf->setAlpha($t['alpha']);

					            $bSetFont = false;

					            if (wpsg_isSizedString($t['font']) && !in_array($t['font'], $this->barCodeFonts)) {

						            $font_file = WPSG_PATH_LIB . 'FPDF_1.81/font/' . $t['font'];

						            if (file_exists($font_file)) {

							            include $font_file;

							            $pdf->AddFont($name, '', $t['font']);

							            $bSetFont = true;
							            $pdf->SetFont($name, '', $t['fontsize']);

						            }

					            }

					            if (!$bSetFont) $pdf->SetFont('Arial', 'B', $t['fontsize']);

					            if (in_array($t['font'], $this->barCodeFonts)) {

						            $font_function = $t['font'];

						            $pdf->SetDrawColor(0, 0, 0);
						            $pdf->$font_function($t['x'], $t['y'], $t['text'], $t['fontsize']);

					            } else {

						            $pdf->wpsg_SetTextColor($t["color"]);

						            $pdf->RotatedText(
							            $t['x'], $t['y'],
							            $t['text'],
							            $t['angle'],
							            $t['align']
						            );

						            $pdf->setAlpha(1);

					            }


				            }

			            }

		            }

	            }

	            // Daten in die Individualisierung eintragen
	            $this->db->UpdateQuery(WPSG_TBL_PDFINDIV, array(
	                'fielddata' => wpsg_q(serialize($fielddata))
	            ), " `id` = '".wpsg_q($indiv_id)."' ");

	            $pdf->Output($trg_file, 'f');

	            chmod($trg_file, 0777);

	        }
	        else {

	            return false;

	        }

	    } // private function indivPDF($src_file, $trg_file)

	    public function template_redirect()
	    {

	        if (wpsg_isSizedString($_REQUEST['wpsg_action'], 'wpsg_mod_downloadplus_download'))
	        {

	            if (!isset($_REQUEST['hash']) || trim($_REQUEST['hash']) == '') die(__('Kein Kontrollhash!', 'wpsg'));

	            $hash = $this->makeHash($_REQUEST['indi']);

	            if ($_REQUEST['hash'] != $hash) die(__('Ungültiger Kontrollhash!', 'wpsg'));

	            $indiv_db = $this->db->fetchRow("SELECT * FROM `".WPSG_TBL_PDFINDIV."` WHERE `id` = '".wpsg_q($_REQUEST['indi'])."'");

	            if ($this->shop->get_option('wpsg_mod_downloadplus_accepted') == '1' && $indiv_db['counter'] >= $this->shop->get_option('wpsg_mod_downloadplus_downloads'))
	            {
	                die(__('Sie haben die maximale Anzahl an Downloads überschritten.', 'wpsg'));
	            }

	            if ($this->shop->get_option('wpsg_mod_downloadplus_accepted') == '2' && time() > (strtotime($indiv_db['done']) + ($this->shop->get_option('wpsg_mod_downloadplus_days') * 86400)))
	            {
	                die(__('Der Download der Datei ist nicht mehr gültig.', 'wpsg'));
	            }

	            $this->db->UpdateQuery(WPSG_TBL_PDFINDIV, array("counter" => wpsg_q($indiv_db['counter'] + 1)), "`id` = '".wpsg_q($_REQUEST['indi'])."'");

	            if ($this->shop->get_option('wpsg_mod_downloadplus_format') == '1')
	            {

	                if ($this->shop->get_option('wpsg_mod_downloadplus_memorylimit') == '1') ini_set('memory_limit', '2000M');
	                if ($this->shop->get_option('wpsg_mod_downloadplus_timelimit') == '1') set_time_limit(3600);

	                // Gezippt intern
	                $zipfile = tempnam("tmp", "zip");
	                $zip = new ZipArchive();
	                $zip->open($zipfile, ZipArchive::CREATE);
	                $zip->addFile($this->getPDFIndivPath($indiv_db['o_id']).'/'.rawurldecode($indiv_db['out_file']), $indiv_db['out_file']);
	                $zip->close();

	                $pi_file = pathinfo($this->getPDFIndivPath($indiv_db['o_id']).'/'.rawurldecode($indiv_db['out_file']));
	                $zip_name = $pi_file['filename'].".zip";

	                wpsg_ob_end_clean();

	                header('Content-Type: application/zip');
	                header('Content-Length: '.filesize($zipfile));
	                header('Content-Disposition: attachment; filename="'.$zip_name.'"');
	                header('Expires: 0');
	                header('Cache-Control: must-revalidate, post-check=0, pre-check=0');
	                header('Pragma: public');

	                readfile($zipfile);
	                unlink($zipfile);
	                die();

	            }
	            /*else if ($this->shop->get_option('wpsg_mod_downloadplus_format') == '1')
	            {

	                // Eventuell TODO

	            }*/
	            else
	            {

	                wpsg_ob_end_clean();

	                header('Content-Type: application/pdf');
	                header('Content-Disposition: inline; filename="'.rawurldecode($indiv_db['out_file']).'"');
	                header('Cache-Control: private, max-age=0, must-revalidate');
	                header('Pragma: public');

	                //header('Content-Disposition: attachment; filename="'.rawurldecode($indiv_db['out_file']).'"');
	                die(file_get_contents($this->getPDFIndivPath($indiv_db['o_id']).'/'.rawurldecode($indiv_db['out_file'])));

	            }

	        }
	        else if (wpsg_isSizedString($_REQUEST['wpsg_action'], 'wpsg_mod_downloadplus_send'))
	        {

	            if (!isset($_REQUEST['hash']) || trim($_REQUEST['hash']) == '') die(__('Kein Kontrollhash!', 'wpsg'));

	            $order = $this->shop->cache->loadOrder($_REQUEST['order']);

	            $hash = md5($order['cdate'].'wpsg'.$order['id']);

	            if ($hash != $_REQUEST['hash']) die(__('Ungültiger Kontrollhash!', 'wpsg'));

	            $this->sendFiles($_REQUEST['order']);

	            die(__('Mail mit Downloadlinks versendet!', 'wpsg'));

	        }

	    }

	    public function cron() {

	        $arPDFsUnindiv = $this->db->fetchAssoc("
				SELECT
					*
				FROM
					`".WPSG_TBL_PDFINDIV."`
				WHERE
					`scheduled` != '0000-00-00 00:00:00' AND
					`done` = '0000-00-00 00:00:00'
			");

	        $arLog = array();

	        foreach ($arPDFsUnindiv as $v) {

	            $order_path = $this->getPDFIndivPath($v['o_id']);

	            $product_id = $this->shop->getProduktID($v['p_key']);

	            if ($v['mode'] === '1') $src_path = $this->getPDFTemplatePath($v['template']);
	            else $src_path = $this->getPDFPath($product_id);

	            $oOrder = wpsg_order::getInstance(intval($v['o_id']));

	            foreach ($oOrder->getOrderProducts() as $oOrderProduct) {

	                if ($oOrderProduct->getProductKey() === $v['p_key']) {

	                	$arFiles = $this->getFiles($oOrderProduct->getProductKey());

	                	foreach ($arFiles as $f) {

	                		if (file_exists($src_path.$f)) {

	                			 $this->indivPDF(
			                        $oOrderProduct->getProductKey(),
			                        $src_path.$f,
			                        $this->getPDFIndivPath($v['o_id']).'/'.$v['out_file'],
			                        $v['o_id'],
			                        $v['downloadnr'],
			                        $v['id'],
			                        $v['product_index']
			                    );

			                    if ($this->shop->get_option('wpsg_mod_downloadplus_emptypdf') === '1') {

			                        $this->indivPDF(
				                        $oOrderProduct->getProductKey(),
			                            $src_path.$f,
				                        $this->getPDFIndivPath($v['o_id']).'/clear_'.$v['out_file'],
				                        $v['o_id'],
				                        $v['downloadnr'],
				                        $v['id'],
				                        $v['product_index'],
					                    false
				                    );

			                    }

			                    $this->db->UpdateQuery(WPSG_TBL_PDFINDIV, array(
			                        "done" => "NOW()"
			                    ), "`id` = '".wpsg_q($v['id'])."'");

			                    if (isset($arLog[$v['o_id']])) $arLog[$v['o_id']] ++;
			                    else $arLog[$v['o_id']] = 1;

			                }

		                }

	                }

	            }

	        }

	        $arClean = array();

	        // Alte Downloads säubern
	        if ($this->shop->get_option('wpsg_mod_downloadplus_accepted') == '1' && $this->shop->get_option('wpsg_mod_downloadplus_cleanraiddownloads') == '1') {

	            $arClean = array_merge($arClean, $this->db->fetchAssoc("
					SELECT
						*
					FROM
						`".WPSG_TBL_PDFINDIV."`
					WHERE
						`counter` > '".wpsg_q($this->shop->get_option('wpsg_mod_downloadplus_downloads'))."'
				"));

	        } else if ($this->shop->get_option('wpsg_mod_downloadplus_accepted') == '2' && $this->shop->get_option('wpsg_mod_downloadplus_cleanraiddownloads') == '1') {

	            $arClean = array_merge($arClean, $this->db->fetchAssoc("
					SELECT
						*
					FROMLetzter Lauf
						`".WPSG_TBL_PDFINDIV."`
					WHERE
						DATE_ADD(`done`, INTERVAL ".$this->shop->get_option('wpsg_mod_downloadplus_days')." DAY) < NOW()
				"));

	        }

	        if (intval($this->shop->get_option('wpsg_mod_downloadplus_cleanolddownloads')) > 0) {

	            $arClean = array_merge($arClean, $this->db->fetchAssoc("
						SELECT
							*
						FROM
							`".WPSG_TBL_PDFINDIV."`
						WHERE
							DATE_ADD(`done`, INTERVAL ".$this->shop->get_option('wpsg_mod_downloadplus_cleanolddownloads')." DAY) < NOW()
					"));

	        }

	        if (wpsg_isSizedArray($arClean)) {

	            foreach ($arClean as $c) {

	                $path = $this->getPDFIndivPath($c['o_id']);

	                unlink($path.'/'.$c['out_file']);

	                $this->db->Query("DELETE FROM `".WPSG_TBL_PDFINDIV."` WHERE `id` = '".wpsg_q($c['id'])."'");

	            }

	        }

	        if (wpsg_isSizedArray($arLog)) {

	            $this->shop->view['arOrder'] = $arLog;

	            foreach ($this->shop->view['arOrder'] as $k => $v) {

	                $order = $this->shop->cache->loadOrder($k);

	                $this->shop->view['arOrder'][$k] = array($v, $order, md5($order['cdate'].'wpsg'.$order['id']));

	                if ($this->shop->get_option('wpsg_mod_downloadplus_autosend') == '1') {

	                    $this->sendFiles($order['id']);

	                }

	            }

	            $mail_text = $this->shop->render(WPSG_PATH_VIEW.'/mods/mod_downloadplus/cronmail.phtml', false);

	            if ($this->shop->get_option('wpsg_htmlmail') === '1') {

	                $mail_html = $this->shop->render(WPSG_PATH_VIEW.'/mods/mod_downloadplus/cronmail_html.phtml', false);

	            } else {

	                $mail_html = false;

	            }

	            $this->shop->sendMail($mail_text, $this->shop->get_option('wpsg_pdfdownloadcronmail_empfaenger'), 'pdfdownloadcronmail', array(), false, false, $mail_html);

	        }

	    } // public function cron()

	    public function wpsg_mod_export_loadFields(&$arFields)
	    {

	        $arDPFields = array(
	            'name' => __('Downloadplus', 'wpsg'),
	            'fields' => array()
	        );

	        for ($i = 1; $i <= 10; $i ++)
	        {

	            $arDPFields['fields']['text'.$i] = wpsg_translate(__('Text in Textfeld #1#', 'wpsg'), $i);

	        }

	        $arDPFields['fields']['dnr'] = __('Downloadnummer', 'wpsg');

	        $arFields[] = $arDPFields;

	    } // public function wpsg_mod_export_loadFields(&$arFields)

	    public function wpsg_mod_export_getValue(&$return, $field_value, $o_id, $p_id, $order_product_id, $profil_separator) {

			if (!in_array($field_value, ['dnr']) && !preg_match('/^text\d+$/', $field_value)) return;
			
			$oOrderProduct = wpsg_order_product::getInstance($order_product_id);
			
			$product_index = $oOrderProduct->getProductIndex();
			
	        // Individualisierung heraussuchen
	        $db_indiv = $this->db->fetchRow("SELECT * FROM `".WPSG_TBL_PDFINDIV."` WHERE `o_id` = '".wpsg_q($o_id)."' AND `p_key` = '".wpsg_q($p_id)."' AND `product_index` = '".wpsg_q($product_index)."' ORDER BY `id` DESC LIMIT 1 ");

	        switch ($field_value) {

	            case 'dnr':
	                if (!wpsg_isSizedInt($db_indiv['id'])) $return = ''; else $return = $db_indiv['downloadnr'];
	                break;

	            default:

	                if (preg_match('/^text\d+$/', $field_value)) {

	                    $return = '';

	                    if (wpsg_isSizedInt($db_indiv['id']))
	                    {

	                        $arTexte = unserialize($db_indiv['fielddata']);

	                        if (wpsg_isSizedArray($arTexte))
	                        {

	                            $txt_nr = substr($field_value, 4);

	                            $return = $arTexte['texte'][$txt_nr - 1];

	                        }

	                    }

	                }

	        } // switch

	    } // public function wpsg_mod_export_getValue(&$return, $field_value, $o_id, $p_id, $product_index, $profil_separator)

	    /**
	     * Baut den Kontroll Hash
	     */
	    private function makeHash($indiv_id)
	    {

	        $indiv_db = $this->db->fetchRow("SELECT * FROM `".WPSG_TBL_PDFINDIV."` WHERE `id` = '".wpsg_q($indiv_id)."'");
	        if ($indiv_db['id'] <= 0) return false;

	        $order = $this->shop->cache->loadOrder($indiv_db['o_id']);
	        if ($order['id'] <= 0) return false;

	        $strHash = md5($order['cdate'].'wpsg'.$indiv_db['file']);

	        return $strHash;

	    } // private function makeHash($indiv_id)

	    /**
	     * Sendet die Mail mit den Downloadlinks an den Kunden
	     */
	    public function sendFiles($order_id)
	    {

	        $this->shop->view['arPDFs'] = $this->db->fetchAssoc("
					SELECT
						*
					FROM
						`".WPSG_TBL_PDFINDIV."`
					WHERE
						`o_id` = '".wpsg_q($order_id)."' AND
						`done` != '0000-00-00 00:00:00'
				");

	        foreach ($this->shop->view['arPDFs'] as $k => $v)
	        {

	            // Download noch gültig
	            if ($this->shop->get_option('wpsg_mod_downloadplus_accepted') == '1' && $v['counter'] >= $this->shop->get_option('wpsg_mod_downloadplus_downloads'))
	            {
	                unset($this->shop->view['arPDFs'][$k]);
	            }
	            else if ($this->shop->get_option('wpsg_mod_downloadplus_accepted') == '2' && time() > (strtotime($v['done']) + ($this->shop->get_option('wpsg_mod_downloadplus_days') * 86400)))
	            {
	                unset($this->shop->view['arPDFs'][$k]);
	            }
	            else
	            {
	                $this->shop->view['arPDFs'][$k]['hash'] = $this->makeHash($v['id']);
	            }

	        }

	        if (sizeof($this->shop->view['arPDFs']) > 0)
	        {

	            $this->shop->view['order'] = $this->shop->cache->loadOrder($order_id);
	            $this->shop->view['kunde'] = $this->shop->cache->loadKunden($this->shop->view['order']['k_id']);

	            // Andere Sprache setzten wenn die Bestellung in einer anderen Sprache durchgeführt wurde
	            if (trim($this->shop->view['order']['language']) != '') $this->shop->setTempLocale($this->shop->view['order']['language']);

	            $mail_text = $this->shop->render(WPSG_PATH_VIEW.'/mods/mod_downloadplus/pdfdownloadmail.phtml', false);

	            if ($this->shop->get_option('wpsg_htmlmail') === '1')
	            {

	                $mail_html = $this->shop->render(WPSG_PATH_VIEW.'/mods/mod_downloadplus/pdfdownloadmail_html.phtml', false);

	            }
	            else
	            {

	                $mail_html = false;

	            }

	            $attachment = array();

	            if ($this->shop->get_option('wpsg_mod_downloadplus_attachement') == '1')
	            {

	                foreach ($this->shop->view['arPDFs'] as $k => $v)
	                {

	                    $attachment[] = $this->getPDFIndivPath($order_id).'/'.$v['out_file'];

	                }

	            }

	            $this->shop->sendMail($mail_text, $this->shop->view['kunde']['email'], 'pdfdownloadfiles', $attachment, $order_id, false, $mail_html);

	            $this->shop->restoreTempLocale();

	            $this->db->ImportQuery(WPSG_TBL_OL, array(
	                "o_id" => wpsg_q($order_id),
	                "cdate" => "NOW()",
	                "title" => wpsg_translate(__('Pdf Downloads an:#1#', 'wpsg'), $this->shop->view['kunde']['email']),
	                "mailtext" => wpsg_q($mail_text)
	            ));

	            $this->arMessage[] = __('E-Mail erfolgreich gesendet.', 'wpsg');

	        }
	        else
	        {

	            $this->arError[] = __('Keine gültigen Downloads.', 'wpsg');

	        }

	    } // public function sendFiles($order_id)

	    /**
	     * Individualisiert die PDFs für eine Bestellung
	     */
	    public function indivOrder($order_id) {

	        $arBasket = $this->shop->cache->loadBasketArray($order_id);

	        $bPDFProdukt = false;
	        $arPDF = array();

	        $oOrder = wpsg_order::getInstance($order_id);

	        foreach ($oOrder->getOrderProducts() as $oOrderProduct) {

	        	if ($this->isPDFProdukt($oOrderProduct->getProductKey())) {

	        		$oProduct = $oOrderProduct->getProduct();
	        		$bPDFProdukt = true;
	        		$product_files = $this->getFiles($oOrderProduct->getProductKey());

	        		foreach ($product_files as $file) {

	        			$arPDF[] = array(
	                        $oOrderProduct->getProductKey(), $file, $oOrderProduct->getProductIndex(),
		                    'mode' => $oProduct->getMeta('mod_downloadplus_mode'),
		                    'template' => $this->getTemplateIdFromProduct($oOrderProduct->getProductKey())
	                    );

			        }

		        }

	        }

	        if ($bPDFProdukt === true && wpsg_isSizedArray($arPDF)) {

	            foreach ($arPDF as $v) {

	                $this->db->Query("DELETE FROM `".WPSG_TBL_PDFINDIV."` WHERE `file` = '".wpsg_q($v[1])."' AND `o_id` = '".wpsg_q($order_id)."' AND `p_key` = '".wpsg_q($v[0])."' AND `product_index` = '".wpsg_q($v[2])."' ");

	                $download_nr = $this->getDownloadNr($order_id, $v[0], $v[2]);

	                $this->db->ImportQuery(WPSG_TBL_PDFINDIV, array(
	                    "o_id" => wpsg_q($order_id),
	                    "downloadnr" => wpsg_q($download_nr),
	                    "p_key" => wpsg_q($v[0]),
	                    'product_index' => wpsg_q($v[2]),
	                    "downloadnr" => wpsg_q($download_nr),
	                    "file" => wpsg_q($v[1]),
	                    "scheduled" => wpsg_q("NOW()"),
	                    "done" => wpsg_q("0000-00-00 00:00:00"),
	                    "out_file" => wpsg_q($this->getNewName($v[1], $order_id, $v[0], $download_nr, $v[2])),
		                'mode' => $v['mode'],
		                'template' => $v['template']
	                ));

	            }

	        }

	    } // public function indivOrder($order_id)

	    /**
	     * Gibt den Individualisierten Dateinamen für ein File und Bestellung zurück
	     */
	    public function getNewName($oldFilename, $order_id, $product_key, $download_nr, $product_index)
	    {

	        $product_id = $this->shop->getProduktId($product_key);

	        $arReplace = array(
	            '/%filename%/i' => $oldFilename,
	            '/%filename_clear%/i' => preg_replace('/\.(.*)$/', '', $oldFilename),
	            '/%filename_ohneext%/i' => preg_replace('/\.(.*)$/', '', $oldFilename),
	            '/%download_nr%/i' => $download_nr
	        );

	        $newFilename = $this->shop->replaceUniversalPlatzhalter($this->shop->get_option('wpsg_mod_downloadplus_filename'), $order_id, false, false, $product_id, $arReplace, $product_index);

	        return $newFilename;

	    } // public function getNewName($oldFilename, $order_id, $product_key, $download_nr, $product_index)

	    /**
	     * Gibt die PDFdateien eines Produktes zurück
	     */
	    public function getFiles($produkt_key, $ignoreTemplateFiles = false) {

	        $product_id = $this->shop->getProduktID($produkt_key); $oProduct = wpsg_product::getInstance($product_id);
			$arPDFFiles = [];

	        if (!$ignoreTemplateFiles && $oProduct->getMeta('mod_downloadplus_mode') === '1') {

	        	$template_id = $this->getTemplateIdFromProduct($produkt_key);
	        	$template = $this->getTemplate($template_id)[0];

	        	$path = $this->getPDFTemplatePath($template['id']);

	        	if (wpsg_isSizedString($template['file']) && file_exists($path.$template['file'])) {

	        		$arPDFFiles[] = $template['file'];

		        }

	        } else {

		        $path = $this->getPDFPath($product_id);

		        if (!file_exists($path)) return;

		        $arrFiles = scandir($path);

		        foreach ($arrFiles as $file) {

		            if (is_file($path.$file) && in_array(strtolower(preg_replace('/(.*)\./', '', $file)), $this->valid_ending)) {

		                $arPDFFiles[] = $file;

		            }

		        }

	        }

	        return $arPDFFiles;

	    } // private function getFiles($produkt_id)

	    /**
	     * Gibt alle Dokumente zurück, die zu einer Bestellung individualisiert wurden
	     */
	    public function getIndivFiles($produkt_id, $order_id)
	    {

	        $produkt_id = $this->shop->getProduktID($produkt_id);

	        $arFilesProdukt = $this->getFiles($produkt_id);
	        $arFilesReturn = array();

	        foreach ($arFilesProdukt as $k => $v)
	        {

	            $indiv = $this->db->fetchRow("SELECT * FROM `".WPSG_TBL_PDFINDIV."` WHERE `o_id` = '".wpsg_q($order_id)."' AND `file` = '".wpsg_q($v)."'");

	            $infoArray = array(
	                'done' => '0000-00-00 00:00:00',
	                'scheduled' => '0000-00-00 00:00:00',
	                'counter' => -1,
	                'status' => 0
	            );

	            if ($indiv['id'] > 0)
	            {

	                $infoArray['status'] = 1;
	                $infoArray['done'] = $indiv['done'];
	                $infoArray['scheduled'] = $indiv['scheduled'];
	                $infoArray['counter'] = $indiv['counter'];
	                $infoArray['indiv_id'] = $indiv['id'];
	                $infoArray['hash'] = $this->makeHash($indiv['id']);

	            }

	            if (@strtotime($indiv['done']) > 0) $infoArray['status'] = 2;

	            $infoArray['file'] = $v;
	            $infoArray['file_out'] = @$indiv['out_file'];
	            $infoArray['path'] = $this->getPDFIndivPath($order_id);

	            $arFilesReturn[] = $infoArray;

	        }

	        return $arFilesReturn;

	    } // public function getIndivFiles($produkt_id, $order_id)

	    /**
	     * Überprüft ein Produkt ob es ein PDFDownloadProdukt ist
	     */
	    public function isPDFProdukt($produkt_key) {

	    	$oProduct = wpsg_product::getInstance($this->shop->getProduktID($produkt_key));
	        $arPDFs = $this->getFiles($produkt_key);

	        if (sizeof($arPDFs) > 0 || $oProduct->getMeta('mod_downloadplus_set') === '1') return true;

	        return false;

	    } // private function isPDFProdukt($produkt_key)

	    /**
	     * Gibt den Pfad zu dem Ordner zurück in dem die PDFs abgelegt werden
	     */
	    public function getPDFPath($produkt_id)
	    {

	        if ($this->shop->get_option('wpsg_mod_downloadplus_pdfpath') !== false && trim($this->shop->get_option('wpsg_mod_downloadplus_pdfpath')) != '')
	        {

	            $path = $this->shop->get_option('wpsg_mod_downloadplus_pdfpath').'/'.$produkt_id.'/';

	        }
	        else
	        {

	            if ($this->shop->isMultiBlog())
	            {

	                $path = WP_CONTENT_DIR.'/'.WPSG_MB_UPLOADS.'/wpsg/wpsg_pdfprodukte/'.$produkt_id.'/';

	            }
	            else
	            {

	                $path = WP_CONTENT_DIR.'/uploads/wpsg/wpsg_pdfprodukte/'.$produkt_id.'/';

	            }

	        }

	        $this->shop->protectDirectory($path);

	        return $path;

	    } // public function getPDFPath($produkt_id)

	    /**
	     * Gibt den Pfad zurück wo die Individualisierten PDFs einer Bestellung liegen
	     */
	    public function getPDFIndivPath($order_id) {

	        if ($this->shop->get_option('wpsg_mod_downloadplus_pdfindivpath') !== false && trim($this->shop->get_option('wpsg_mod_downloadplus_pdfindivpath')) != '') {

	            $path = $this->shop->get_option('wpsg_mod_downloadplus_pdfindivpath').'/'.$order_id.'/';

	        } else {

	            if ($this->shop->isMultiBlog()) {

	                $path = WP_CONTENT_DIR.'/'.WPSG_MB_UPLOADS.'/wpsg/wpsg_pdfprodukte_order/'.$order_id.'/';

	            } else {

	                $path = WP_CONTENT_DIR.'/uploads/wpsg/wpsg_pdfprodukte_order/'.$order_id.'/';

	            }

	        }

	        $this->shop->protectDirectory($path);

	        return $path;

	    } // public function getPDFIndivPath($order_id)

		/**
		 * Gibt den Pfad zurück, wo die Datei des Templates abgelegt wird
		 * @param $template_id
		 */
		public function getPDFTemplatePath($template_id) {

			$template_id = intval($template_id);

		    if ($this->shop->isMultiBlog()) {

                $path = WP_CONTENT_DIR.'/'.WPSG_MB_UPLOADS.'/wpsg/wpsg_dp_templatefiles/'.$template_id.'/';

            } else {

                $path = WP_CONTENT_DIR.'/uploads/wpsg/wpsg_dp_templatefiles/'.$template_id.'/';

            }

		    if (!file_exists($path)) mkdir($path, 0775, true);

	        return $path;

		}

	    /**
	     * Gibt die DownloadNr zurück
	     * @param int $order_id BestellID
	     * @param varchar $product_key Produktkey
	     */
	    private function getDownloadNr($order_id, $product_key, $product_index)
	    {

	        $product_id = $this->shop->getProduktID($product_key);

	        $format = $this->shop->get_option('wpsg_mod_downloadplus_numberformat');
	        if (!wpsg_isSizedString($format)) $format = "%download_nr%";

	        $download_nr = $this->shop->get_option('wpsg_mod_downloadplus_startnumber');
	        if (!wpsg_isSizedInt($download_nr)) $download_nr = 1;

	        $arCustomReplace = array(
	            '/\%download_nr\%/i' => $download_nr
	        );

	        $format = $this->shop->replaceUniversalPlatzhalter($format, $order_id, false, false, $product_id, $arCustomReplace, $product_index);

	        $this->shop->update_option('wpsg_mod_downloadplus_startnumber', $download_nr + 1);

	        return $format;

	    } // private function getDownloadNr($order_id, $product_key)

		/**
		 * Gibt einen Array mit Templates zurück, wird $template_id übergeben, so wird nur das Template mit dieser ID geladen
		 * @param $template_id
		 */
		public function getTemplate($template_id = null) {

			$strQueryWHERE = " T.`deleted` != '1' ";

			if (wpsg_isSizedInt($template_id)) {

				$strQueryWHERE .= " AND T.`id` = '".wpsg_q($template_id)."' ";

			}

			$arTemplates = $this->db->fetchAssoc("
				SELECT
					T.*
				FROM	
					`".WPSG_TBL_DP_TEMPLATES."` AS T
				WHERE
					".$strQueryWHERE."
			");

			array_walk($arTemplates, function(&$e) {

				$e['id'] = intval($e['id']);
		        $e['defaulttemplate'] = intval($e['defaulttemplate']) > 0;
		        $e['arField'] = $this->getTemplateFields($e['id']);

		        $e['file'] = null;

		        $path = $this->getPDFTemplatePath($e['id']);
		        $arFiles = scandir($path);

		        foreach ($arFiles as $f) {

		        	if (preg_match('/\.pdf$/', $f)) {

		        		$e['file'] = $f;

			        }

		        }

		        $e['file_data'] = null;
		        $e['file_delete'] = false;

			});

			return $arTemplates;

		}

		public function getTemplateIdFromProduct($product_key) {

			$product_id = $this->shop->getProduktID($product_key);
			$oProduct = wpsg_product::getInstance($product_id);

			$product_template = intval($oProduct->getMeta('mod_downloadplus_template'));

			if ($product_template <= 0) {

				// Standard Template
				$template_id = intval($this->db->fetchOne("SELECT `id` FROM `".WPSG_TBL_DP_TEMPLATES."` WHERE `defaulttemplate` = '1' "));

			} else $template_id = $product_template;

			if ($this->shop->hasMod('wpsg_mod_productvariants') && $this->shop->callMod('wpsg_mod_productvariants', 'isVariantsProductKey', [$product_key])) {

				$arVarSet = $this->shop->callMod('wpsg_mod_productvariants', 'getSetVariArray', [$product_key]);

				$vari_template = json_decode($oProduct->getMeta('mod_downloadplus_vari_template'), true);

				if (wpsg_isSizedArray($vari_template)) {

					foreach ($arVarSet as $var_id => $vari_id) {

						$vari_template_id = $this->db->fetchOne("SELECT `template_id` FROM `".WPSG_TBL_VARIANTS_VARI."` WHERE `id` = '".wpsg_q($vari_id)."' ");

						if (wpsg_isSizedInt($vari_template[$var_id.'_'.$vari_id])) $template_id = $vari_template[$var_id.'_'.$vari_id];
						else if ($vari_template_id > 0) { $template_id = $vari_template_id; }

					}

				}

			}

			return $template_id;

		}

		public function getTemplateFields($template_id = false) {

	    	if ($template_id === false) $template_id = $this->db->fetchOne("SELECT `id` FROM `".WPSG_TBL_DP_TEMPLATES."` WHERE `defaulttemplate` = '1' ");

	    	$template_id = intval($template_id);

	    	if ($template_id > 0) {

	    		$arField = $this->db->fetchAssoc("SELECT * FROM `".WPSG_TBL_DP_TEMPLATES_FIELD."` WHERE `template_id` = '".wpsg_q($template_id)."' ");

	    		array_walk($arField, function(&$e) {

	    			$e['id'] = intval($e['id']);
					$e['x'] = intval($e['x']);
					$e['y'] = intval($e['y']);
					$e['align'] = intval($e['align']) > 0;
					$e['bg'] = intval($e['bg']) > 0;
					$e['alpha'] = wpsg_ff(wpsg_tf($e['alpha']));
					$e['angle'] = intval($e['angle']);
					$e['fontsize'] = intval($e['fontsize']);
					$e['aktiv'] = intval($e['aktiv']) > 0;

			    });

	    		return $arField;

		    } else return [];

		}

	} // class wpsg_mod_downloadplus extends wpsg_mod_basic

